package de.gedoplan.beantrial.entitygraph.entity;

import java.util.List;

import javax.persistence.EntityGraph;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.PersistenceUnitUtil;
import javax.persistence.TypedQuery;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

public class PublisherTest
{
  private static EntityManagerFactory entityManagerFactory;
  private EntityManager               entityManager;

  @BeforeClass
  public static void beforeClass()
  {
    Publisher publisher = new Publisher("Expert Press", "Java");
    Book book = new Book("Entity Graphs Investigated", "12345678", 45);
    book.setPublisher(publisher);

    entityManagerFactory = Persistence.createEntityManagerFactory("default");
    EntityManager em = entityManagerFactory.createEntityManager();
    em.getTransaction().begin();
    em.persist(publisher);
    em.getTransaction().commit();
    em.close();
  }

  @Before
  public void before()
  {
    this.entityManager = entityManagerFactory.createEntityManager();
  }

  @After
  public void after()
  {
    this.entityManager.close();
  }

  @Test
  public void testSimpleFetchGraph()
  {
    System.out.println("----- testSimpleFetchGraph -----");
    testEntityGraph("Publisher_books", "javax.persistence.fetchgraph");
  }

  @Test
  public void testSimpleLoadGraph()
  {
    System.out.println("----- testSimpleLoadGraph -----");
    testEntityGraph("Publisher_books", "javax.persistence.loadgraph");
  }

  private void testEntityGraph(String graphName, String hintName)
  {
    PersistenceUnitUtil persistenceUnitUtil = entityManagerFactory.getPersistenceUnitUtil();

    TypedQuery<Publisher> query = this.entityManager.createQuery("select p from Publisher p", Publisher.class);

    EntityGraph<?> entityGraph = this.entityManager.getEntityGraph(graphName);
    Assert.assertNotNull("Entity Graph", entityGraph);

    query.setHint(hintName, entityGraph);

    System.out.println(hintName + " = " + graphName);

    for (Publisher publisher : query.getResultList())
    {
      boolean booksLoaded = persistenceUnitUtil.isLoaded(publisher, "books");
      boolean categoriesLoaded = persistenceUnitUtil.isLoaded(publisher, "categories");
      System.out.println(publisher.toDebugString() + ", books loaded: " + booksLoaded + ", categories loaded: " + categoriesLoaded);

      List<Book> books = publisher.getBooks();
      System.out.println("  books: " + books);
      if (books != null)
      {
        System.out.println("  #books: " + books.size());
        for (Book book : books)
        {
          System.out.println("  " + book.toDebugString());
        }
      }

      List<String> categories = publisher.getCategories();
      System.out.println("  categories: " + categories);
      if (categories != null)
      {
        System.out.println("  #categories: " + categories.size());
        for (String category : categories)
        {
          System.out.println("  " + category);
        }
      }
    }
  }

}
